@@ -15,31 +15,47 @@ class CompositorVsyncDispatcher;
 
 namespace widget {
 
-class CompositorWidgetChild final
- : public PCompositorWidgetChild,
-   public CompositorWidgetDelegate
-{
-public:
+namespace remote_backbuffer {
+class Provider;
+}
+
+class CompositorWidgetChild final : public PCompositorWidgetChild,
+                                    public PlatformCompositorWidgetDelegate {
+ public:
+  NS_INLINE_DECL_THREADSAFE_REFCOUNTING(CompositorWidgetChild, override)
+
   CompositorWidgetChild(RefPtr<CompositorVsyncDispatcher> aVsyncDispatcher,
-                        RefPtr<CompositorWidgetVsyncObserver> aVsyncObserver);
-  ~CompositorWidgetChild() override;
+                        RefPtr<CompositorWidgetVsyncObserver> aVsyncObserver,
+                        const CompositorWidgetInitData& aInitData);
+
+  bool Initialize();
 
   void EnterPresentLock() override;
   void LeavePresentLock() override;
   void OnDestroyWindow() override;
-  void UpdateTransparency(nsTransparencyMode aMode) override;
-  void ClearTransparentWindow() override;
-  HDC GetTransparentDC() const override;
+  bool OnWindowResize(const LayoutDeviceIntSize& aSize) override;
+  void NotifyVisibilityUpdated(bool aIsFullyOccluded) override;
+
+  void UpdateTransparency(TransparencyMode) override;
 
   mozilla::ipc::IPCResult RecvObserveVsync() override;
   mozilla::ipc::IPCResult RecvUnobserveVsync() override;
+  mozilla::ipc::IPCResult RecvUpdateCompositorWnd(
+      const WindowsHandle& aCompositorWnd, const WindowsHandle& aParentWnd,
+      UpdateCompositorWndResolver&& aResolve) override;
 
-private:
+ private:
+  ~CompositorWidgetChild() override;
   RefPtr<CompositorVsyncDispatcher> mVsyncDispatcher;
   RefPtr<CompositorWidgetVsyncObserver> mVsyncObserver;
+  HWND mCompositorWnd;
+
+  HWND mWnd;
+
+  std::unique_ptr<remote_backbuffer::Provider> mRemoteBackbufferProvider;
 };
 
-} // namespace widget
-} // namespace mozilla
+}  // namespace widget
+}  // namespace mozilla
 
-#endif // widget_windows_CompositorWidgetChild_h
+#endif  // widget_windows_CompositorWidgetChild_h
