@@ -11,7 +11,6 @@
 #include <shobjidl.h>
 #include <uxtheme.h>
 #include <dwmapi.h>
-#include <unordered_map>
 #include <utility>
 
 // Undo the windows.h damage
@@ -33,12 +32,9 @@
 #include "nsIDownloader.h"
 #include "nsIURI.h"
 #include "nsIWidget.h"
-#include "nsIThread.h"
+#include "nsWindowsHelpers.h"
 
-#include "mozilla/Attributes.h"
 #include "mozilla/EventForwards.h"
-#include "mozilla/HalScreenConfiguration.h"
-#include "mozilla/HashTable.h"
 #include "mozilla/LazyIdleThread.h"
 #include "mozilla/UniquePtr.h"
 #include "mozilla/Vector.h"
@@ -48,11 +44,13 @@
 
 /**
  * NS_INLINE_DECL_IUNKNOWN_REFCOUNTING should be used for defining and
- * implementing AddRef() and Release() of IUnknown interface.
- * This depends on xpcom/base/nsISupportsImpl.h.
+ * implementing AddRef() and Release() of IUnknown interface and mRefCnt.
+ * NS_INLINE_DECL_IUNKNOWN_ADDREF_RELEASE should be used for overriding
+ * AddRef() and Release() of IUnknown interface.
+ * These depend on xpcom/base/nsISupportsImpl.h.
  */
 
-#define NS_INLINE_DECL_IUNKNOWN_REFCOUNTING(_class)                         \
+#define NS_INLINE_DECL_IUNKNOWN_ADDREF_RELEASE(_class)                      \
  public:                                                                    \
   STDMETHODIMP_(ULONG) AddRef() {                                           \
     MOZ_ASSERT_TYPE_OK_FOR_REFCOUNTING(_class)                              \
@@ -75,11 +73,15 @@
       return 0;                                                             \
     }                                                                       \
     return static_cast<ULONG>(mRefCnt.get());                               \
-  }                                                                         \
-                                                                            \
- protected:                                                                 \
-  nsAutoRefCnt mRefCnt;                                                     \
-  NS_DECL_OWNINGTHREAD                                                      \
+  }
+
+#define NS_INLINE_DECL_IUNKNOWN_REFCOUNTING(_class) \
+ public:                                            \
+  NS_INLINE_DECL_IUNKNOWN_ADDREF_RELEASE(_class)    \
+                                                    \
+ protected:                                         \
+  nsAutoRefCnt mRefCnt;                             \
+  NS_DECL_OWNINGTHREAD                              \
  public:
 
 class nsWindow;
@@ -213,7 +215,6 @@ class WinUtils {
    */
   static double SystemScaleFactor();
 
-  static bool IsPerMonitorDPIAware();
   /**
    * Get the DPI of the given monitor if it's per-monitor DPI aware, otherwise
    * return the system DPI.
@@ -295,34 +296,6 @@ class WinUtils {
    */
   static void WaitForMessage(DWORD aTimeoutMs = INFINITE);
 
-  /**
-   * Gets the value of a string-typed registry value.
-   *
-   * @param aRoot The registry root to search in.
-   * @param aKeyName The name of the registry key to open.
-   * @param aValueName The name of the registry value in the specified key whose
-   *   value is to be retrieved.  Can be null, to retrieve the key's unnamed/
-   *   default value.
-   * @param aBuffer The buffer into which to store the string value.  Can be
-   *   null, in which case the return value indicates just whether the value
-   *   exists.
-   * @param aBufferLength The size of aBuffer, in bytes.
-   * @return Whether the value exists and is a string.
-   */
-  static bool GetRegistryKey(HKEY aRoot, char16ptr_t aKeyName,
-                             char16ptr_t aValueName, wchar_t* aBuffer,
-                             DWORD aBufferLength);
-
-  /**
-   * Checks whether the registry key exists in either 32bit or 64bit branch on
-   * the environment.
-   *
-   * @param aRoot The registry root of aName.
-   * @param aKeyName The name of the registry key to check.
-   * @return TRUE if it exists and is readable.  Otherwise, FALSE.
-   */
-  static bool HasRegistryKey(HKEY aRoot, char16ptr_t aKeyName);
-
   /**
    * GetTopLevelHWND() returns a window handle of the top level window which
    * aWnd belongs to.  Note that the result may not be our window, i.e., it
@@ -336,7 +309,7 @@ class WinUtils {
    * |                         |         TRUE          |         FALSE         |
    + +-----------------+-------+-----------------------+-----------------------+
    * |                 |       |  * an independent top level window            |
-   * |                 | TRUE  |  * a pupup window (WS_POPUP)                  |
+   * |                 | TRUE  |  * a popup window (WS_POPUP)                  |
    * |                 |       |  * an owned top level window (like dialog)    |
    * | aStopIfNotChild +-------+-----------------------+-----------------------+
    * |                 |       |  * independent window | * only an independent |
@@ -407,19 +380,6 @@ class WinUtils {
     return (aLParam & 0x1000000) != 0;
   }
 
-  /**
-   * GetInternalMessage() converts a native message to an internal message.
-   * If there is no internal message for the given native message, returns
-   * the native message itself.
-   */
-  static UINT GetInternalMessage(UINT aNativeMessage);
-
-  /**
-   * GetNativeMessage() converts an internal message to a native message.
-   * If aInternalMessage is a native message, returns the native message itself.
-   */
-  static UINT GetNativeMessage(UINT aInternalMessage);
-
   /**
    * GetMouseInputSource() returns a pointing device information.  The value is
    * one of MouseEvent_Binding::MOZ_SOURCE_*.  This method MUST be called during
@@ -444,6 +404,8 @@ class WinUtils {
    * returns the LayoutDeviceIntRegion.
    */
   static LayoutDeviceIntRegion ConvertHRGNToRegion(HRGN aRgn);
+  /** Performs the inverse of ConvertHRGNToRegion. */
+  static nsAutoRegion RegionToHRGN(const LayoutDeviceIntRegion&);
 
   /**
    * ToIntRect converts a Windows RECT to a LayoutDeviceIntRect.
@@ -452,6 +414,8 @@ class WinUtils {
    * returns the LayoutDeviceIntRect.
    */
   static LayoutDeviceIntRect ToIntRect(const RECT& aRect);
+  /** Performs the inverse of ToIntRect */
+  static RECT ToWinRect(const LayoutDeviceIntRect& aRect);
 
   /**
    * Returns true if the context or IME state is enabled.  Otherwise, false.
@@ -489,9 +453,11 @@ class WinUtils {
   static PointerCapabilities GetPrimaryPointerCapabilities();
   // For any-pointer and any-hover media queries features.
   static PointerCapabilities GetAllPointerCapabilities();
-  // Returns a string containing a comma-separated list of Fluent IDs
-  // representing the currently active pointing devices
-  static void GetPointerExplanation(nsAString* aExplanation);
+
+  // Returns whether the system has any active device for each pointer type.
+  static bool SystemHasMouse();
+  static bool SystemHasTouch();
+  static bool SystemHasPen();
 
   /**
    * Fully resolves a path to its final path name. So if path contains
@@ -581,6 +547,11 @@ class WinUtils {
 
   static bool GetClassName(HWND aHwnd, nsAString& aName);
 
+  static bool MicaAvailable();
+  static bool MicaEnabled();
+  static bool MicaPopupsEnabled();
+  static void UpdateMicaInAllWindows();
+
   static void EnableWindowOcclusion(const bool aEnable);
 
   static bool GetTimezoneName(wchar_t* aBuffer);
@@ -592,6 +563,12 @@ class WinUtils {
 
   static bool GetAutoRotationState(AR_STATE* aRotationState);
 
+  static void GetClipboardFormatAsString(UINT aFormat, nsAString& aOutput);
+
+  static nsresult GetProcessImageName(DWORD aProcessId, nsAString& aName);
+
+  static void InvalidateWindowPreviews();
+
  private:
   static WhitelistVec BuildWhitelist();
 
@@ -601,26 +578,7 @@ class WinUtils {
 #endif
 };
 
-#ifdef MOZ_PLACES
-class AsyncFaviconDataReady final : public nsIFaviconDataCallback {
- public:
-  NS_DECL_ISUPPORTS
-  NS_DECL_NSIFAVICONDATACALLBACK
-
-  AsyncFaviconDataReady(nsIURI* aNewURI, RefPtr<LazyIdleThread>& aIOThread,
-                        const bool aURLShortcut,
-                        already_AddRefed<nsIRunnable> aRunnable);
-  nsresult OnFaviconDataNotAvailable(void);
-
- private:
-  ~AsyncFaviconDataReady() {}
-
-  nsCOMPtr<nsIURI> mNewURI;
-  RefPtr<LazyIdleThread> mIOThread;
-  nsCOMPtr<nsIRunnable> mRunnable;
-  const bool mURLShortcut;
-};
-#endif
+typedef MozPromise<nsString, nsresult, true> ObtainCachedIconFileAsyncPromise;
 
 /**
  * Asynchronously tries add the list to the build
@@ -632,10 +590,12 @@ class AsyncEncodeAndWriteIcon : public nsIRunnable {
 
   // Warning: AsyncEncodeAndWriteIcon assumes ownership of the aData buffer
   // passed in
-  AsyncEncodeAndWriteIcon(const nsAString& aIconPath,
-                          UniquePtr<uint8_t[]> aData, uint32_t aStride,
-                          uint32_t aWidth, uint32_t aHeight,
-                          already_AddRefed<nsIRunnable> aRunnable);
+  AsyncEncodeAndWriteIcon(
+      const nsAString& aIconPath, UniquePtr<uint8_t[]> aData, uint32_t aStride,
+      uint32_t aWidth, uint32_t aHeight,
+      already_AddRefed<nsIRunnable> aRunnable,
+      UniquePtr<MozPromiseHolder<ObtainCachedIconFileAsyncPromise>>
+          aPromiseHolder = nullptr);
 
  private:
   virtual ~AsyncEncodeAndWriteIcon();
@@ -643,6 +603,7 @@ class AsyncEncodeAndWriteIcon : public nsIRunnable {
   nsAutoString mIconPath;
   UniquePtr<uint8_t[]> mBuffer;
   nsCOMPtr<nsIRunnable> mRunnable;
+  UniquePtr<MozPromiseHolder<ObtainCachedIconFileAsyncPromise>> mPromiseHolder;
   uint32_t mStride;
   uint32_t mWidth;
   uint32_t mHeight;
@@ -665,15 +626,20 @@ class AsyncDeleteAllFaviconsFromDisk : public nsIRunnable {
 
 class FaviconHelper {
  public:
+  enum class IconCacheDir : uint8_t {
+    JumpListCacheDir = 1,
+    ShortcutCacheDir = 2
+  };
+
   static const char kJumpListCacheDir[];
   static const char kShortcutCacheDir[];
   static nsresult ObtainCachedIconFile(
       nsCOMPtr<nsIURI> aFaviconPageURI, nsString& aICOFilePath,
       RefPtr<LazyIdleThread>& aIOThread, bool aURLShortcut,
       already_AddRefed<nsIRunnable> aRunnable = nullptr);
-
-  static nsresult HashURI(nsCOMPtr<nsICryptoHash>& aCryptoHash, nsIURI* aUri,
-                          nsACString& aUriHash);
+  static RefPtr<ObtainCachedIconFileAsyncPromise> ObtainCachedIconFileAsync(
+      nsCOMPtr<nsIURI> aFaviconPageURI, RefPtr<LazyIdleThread>& aIOThread,
+      IconCacheDir aCacheDir);
 
   static nsresult GetOutputIconPath(nsCOMPtr<nsIURI> aFaviconPageURI,
                                     nsCOMPtr<nsIFile>& aICOFile,
@@ -681,8 +647,10 @@ class FaviconHelper {
 
   static nsresult CacheIconFileFromFaviconURIAsync(
       nsCOMPtr<nsIURI> aFaviconPageURI, nsCOMPtr<nsIFile> aICOFile,
-      RefPtr<LazyIdleThread>& aIOThread, bool aURLShortcut,
-      already_AddRefed<nsIRunnable> aRunnable);
+      RefPtr<nsISerialEventTarget> aIOThread, bool aURLShortcut,
+      already_AddRefed<nsIRunnable> aRunnable,
+      UniquePtr<MozPromiseHolder<ObtainCachedIconFileAsyncPromise>>
+          aPromiseHolder);
 
   static int32_t GetICOCacheSecondsTimeout();
 };
@@ -691,13 +659,15 @@ MOZ_MAKE_ENUM_CLASS_BITWISE_OPERATORS(WinUtils::PathTransformFlags);
 
 // RTL shim windows are temporary child windows of our nsWindows created to
 // address RTL issues in picker dialogs. (See bug 588735.)
-class MOZ_STACK_CLASS ScopedRtlShimWindow {
+class ScopedRtlShimWindow {
  public:
   explicit ScopedRtlShimWindow(nsIWidget* aParent);
   ~ScopedRtlShimWindow();
 
   ScopedRtlShimWindow(const ScopedRtlShimWindow&) = delete;
-  ScopedRtlShimWindow(ScopedRtlShimWindow&&) = delete;
+  ScopedRtlShimWindow(ScopedRtlShimWindow&& that) noexcept : mWnd(that.mWnd) {
+    that.mWnd = nullptr;
+  };
 
   HWND get() const { return mWnd; }
 
